@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: graphicsY11.ml,v 1.4 2010-08-28 07:10:45 remy Exp $ *)
+(* $Id: graphicsY11.ml,v 1.5 2011-08-04 13:25:59 remy Exp $ *)
 
 (* Module [GraphicsY11]:
    additional graphics primitives for the X Windows system *)
@@ -496,3 +496,12 @@ let init () =
 external get_button: int -> int = "caml_gr_get_button";;
 
 external button_pressed: unit -> bool = "caml_gr_poll_button_pressed";;
+
+external button_enqueued: int -> bool = "caml_gr_button_enqueued";;
+
+let button_enqueued m =
+  let b = button_enqueued m in
+  Misc.debug_endline (if b then "Queue => Full" else "Queue => Empty");
+  b
+              
+
